@@ -32,6 +32,10 @@
 #include "formats/cvs2cl.h"
 #include "formats/svn.h"
 
+#ifndef GOURCE_FONT_FILE
+#define GOURCE_FONT_FILE "FreeSans.ttf"
+#endif
+
 GourceSettings gGourceSettings;
 
 //display help message
@@ -39,7 +43,7 @@ void GourceSettings::help(bool extended_help) {
 
 #ifdef _WIN32
     //resize window to fit help message
-    SDLApp::resizeConsole(780);
+    SDLApp::resizeConsole(1000);
     SDLApp::showConsole(true);
 #endif
 
@@ -53,8 +57,6 @@ void GourceSettings::help(bool extended_help) {
     printf("      --screen SCREEN              Screen number\n");
     printf("      --multi-sampling             Enable multi-sampling\n");
     printf("      --no-vsync                   Disable vsync\n\n");
-    printf("  --window-position XxY            Initial window position\n");
-    printf("  --frameless                      Frameless window\n\n");
 
     printf("  --start-date 'YYYY-MM-DD hh:mm:ss +tz'  Start at a date and optional time\n");
     printf("  --stop-date  'YYYY-MM-DD hh:mm:ss +tz'  Stop at a date and optional time\n\n");
@@ -70,6 +72,8 @@ void GourceSettings::help(bool extended_help) {
     printf("      --disable-auto-skip          Disable auto skip\n");
     printf("  -s, --seconds-per-day SECONDS    Speed in seconds per day (default: 10)\n");
     printf("      --realtime                   Realtime playback speed\n");
+    printf("      --no-time-travel             Use the time of the last commit if the\n");
+    printf("                                   time of a commit is in the past\n");
     printf("  -c, --time-scale SCALE           Change simulation time scale (default: 1.0)\n");
     printf("  -e, --elasticity FLOAT           Elasticity of nodes (default: 0.0)\n\n");
 
@@ -96,6 +100,9 @@ void GourceSettings::help(bool extended_help) {
 if(extended_help) {
     printf("Extended Options:\n\n");
 
+    printf("  --window-position XxY    Initial window position\n");
+    printf("  --frameless              Frameless window\n\n");
+
     printf("  --output-custom-log FILE  Output a custom format log file ('-' for STDOUT).\n\n");
 
     printf("  -b, --background-colour  FFFFFF    Background colour in hex\n");
@@ -110,12 +117,21 @@ if(extended_help) {
 
     printf("  --disable-auto-rotate    Disable automatic camera rotation\n\n");
 
+    printf("  --disable-input          Disable keyboard and mouse input\n\n");
+
     printf("  --date-format FORMAT     Specify display date string (strftime format)\n\n");
 
+    printf("  --font-file FILE         Specify the font\n");
+    printf("  --font-scale SCALE       Scale the size of all fonts\n");
     printf("  --font-size SIZE         Font size used by date and title\n");
+    printf("  --file-font-size SIZE    Font size for filenames\n");
+    printf("  --dir-font-size SIZE     Font size for directory names\n");
+    printf("  --user-font-size SIZE    Font size for user names\n");
     printf("  --font-colour FFFFFF     Font colour used by date and title in hex\n\n");
 
-    printf("  --file-extensions        Show filename extensions only\n\n");
+    printf("  --file-extensions          Show filename extensions only\n");
+    printf("  --file-extension-fallback  Use filename as extension if the extension\n");
+    printf("                             is missing or empty\n\n");
 
     printf("  --git-branch             Get the git log of a particular branch\n\n");
 
@@ -125,11 +141,14 @@ if(extended_help) {
     printf("  --logo IMAGE             Logo to display in the foreground\n");
     printf("  --logo-offset XxY        Offset position of the logo\n\n");
 
+    printf("  --loop-delay-seconds SECONDS Seconds to delay before looping (default: 3)\n\n");
+
     printf("  --title TITLE            Set a title\n\n");
 
     printf("  --transparent            Make the background transparent\n\n");
 
     printf("  --user-filter REGEX      Ignore usernames matching this regex\n");
+    printf("  --user-show-filter REGEX Show only usernames matching this regex\n\n");
     printf("  --file-filter REGEX      Ignore file paths matching this regex\n");
     printf("  --file-show-filter REGEX Show only file paths matching this regex\n\n");
 
@@ -147,7 +166,9 @@ if(extended_help) {
     printf("  --filename-colour        Font colour for filenames.\n");
     printf("  --dir-colour             Font colour for directories.\n\n");
 
-    printf("  --dir-name-depth DEPTH   Draw names of directories down to a specific depth.\n\n");
+    printf("  --dir-name-depth DEPTH    Draw names of directories down to a specific depth.\n");
+    printf("  --dir-name-position FLOAT Position along edge of the directory name\n");
+    printf("                            (between 0.0 and 1.0, default is 0.5).\n\n");
 
     printf("  --filename-time SECONDS  Duration to keep filenames on screen (default: 4.0)\n\n");
 
@@ -223,33 +244,36 @@ GourceSettings::GourceSettings() {
     conf_sections["log-level"]         = "command-line";
 
     //boolean args
-    arg_types["help"]            = "bool";
-    arg_types["extended-help"]   = "bool";
-    arg_types["stop-on-idle"]    = "bool";
-    arg_types["stop-at-end"]     = "bool";
-    arg_types["dont-stop"]       = "bool";
-    arg_types["loop"]            = "bool";
-    arg_types["realtime"]        = "bool";
-    arg_types["colour-images"]   = "bool";
-    arg_types["hide-date"]       = "bool";
-    arg_types["hide-files"]      = "bool";
-    arg_types["hide-users"]      = "bool";
-    arg_types["hide-tree"]       = "bool";
-    arg_types["hide-usernames"]  = "bool";
-    arg_types["hide-filenames"]  = "bool";
-    arg_types["hide-dirnames"]   = "bool";
-    arg_types["hide-progress"]   = "bool";
-    arg_types["hide-bloom"]      = "bool";
-    arg_types["hide-mouse"]      = "bool";
-    arg_types["hide-root"]       = "bool";
-    arg_types["highlight-users"] = "bool";
-    arg_types["highlight-dirs"]  = "bool";
-    arg_types["file-extensions"] = "bool";
-    arg_types["key"]             = "bool";
-    arg_types["ffp"]             = "bool";
+    arg_types["help"]                    = "bool";
+    arg_types["extended-help"]           = "bool";
+    arg_types["stop-on-idle"]            = "bool";
+    arg_types["stop-at-end"]             = "bool";
+    arg_types["dont-stop"]               = "bool";
+    arg_types["loop"]                    = "bool";
+    arg_types["realtime"]                = "bool";
+    arg_types["no-time-travel"]          = "bool";
+    arg_types["colour-images"]           = "bool";
+    arg_types["hide-date"]               = "bool";
+    arg_types["hide-files"]              = "bool";
+    arg_types["hide-users"]              = "bool";
+    arg_types["hide-tree"]               = "bool";
+    arg_types["hide-usernames"]          = "bool";
+    arg_types["hide-filenames"]          = "bool";
+    arg_types["hide-dirnames"]           = "bool";
+    arg_types["hide-progress"]           = "bool";
+    arg_types["hide-bloom"]              = "bool";
+    arg_types["hide-mouse"]              = "bool";
+    arg_types["hide-root"]               = "bool";
+    arg_types["highlight-users"]         = "bool";
+    arg_types["highlight-dirs"]          = "bool";
+    arg_types["file-extensions"]         = "bool";
+    arg_types["file-extension-fallback"] = "bool";
+    arg_types["key"]                     = "bool";
+    arg_types["ffp"]                     = "bool";
 
     arg_types["disable-auto-rotate"] = "bool";
     arg_types["disable-auto-skip"]   = "bool";
+    arg_types["disable-input"]       = "bool";
 
     arg_types["git-log-command"]= "bool";
     arg_types["cvs-exp-command"]= "bool";
@@ -268,17 +292,23 @@ GourceSettings::GourceSettings() {
     arg_types["user-friction"]     = "float";
     arg_types["padding"]           = "float";
     arg_types["time-scale"]        = "float";
+    arg_types["dir-name-position"] = "float";
+    arg_types["loop-delay-seconds"] = "float";
 
     arg_types["max-files"] = "int";
     arg_types["font-size"] = "int";
+    arg_types["font-scale"] = "float";
+    arg_types["file-font-size"] = "int";
+    arg_types["dir-font-size"] = "int";
+    arg_types["user-font-size"] = "int";
     arg_types["hash-seed"] = "int";
 
-    arg_types["user-filter"]    = "multi-value";
-    arg_types["follow-user"]    = "multi-value";
-    arg_types["highlight-user"] = "multi-value";
-
+    arg_types["user-filter"]      = "multi-value";
+    arg_types["user-show-filter"] = "multi-value";
     arg_types["file-filter"]      = "multi-value";
     arg_types["file-show-filter"] = "multi-value";
+    arg_types["follow-user"]      = "multi-value";
+    arg_types["highlight-user"]   = "multi-value";
 
     arg_types["log-level"]          = "string";
     arg_types["background-image"]   = "string";
@@ -307,6 +337,7 @@ GourceSettings::GourceSettings() {
     arg_types["user-scale"]         = "string";
     arg_types["camera-mode"]        = "string";
     arg_types["title"]              = "string";
+    arg_types["font-file"]          = "string";
     arg_types["font-colour"]        = "string";
     arg_types["highlight-colour"]   = "string";
     arg_types["selection-colour"]   = "string";
@@ -355,17 +386,21 @@ void GourceSettings::setGourceDefaults() {
     stop_on_idle   = false;
     stop_at_end    = false;
     dont_stop      = false;
+    no_time_travel = false;
 
     show_key = false;
 
     disable_auto_rotate = false;
 
+    disable_input = false;
+
     auto_skip_seconds = 3.0f;
     days_per_second   = 0.1f; // TODO: check this is right
     file_idle_time    = 0.0f;
     time_scale        = 1.0f;
 
     loop = false;
+    loop_delay_seconds = 3.0f;
 
     logo = "";
     logo_offset = vec2(20.0f,20.0f);
@@ -393,13 +428,18 @@ void GourceSettings::setGourceDefaults() {
 
     title             = "";
 
+    font_file = GOURCE_FONT_FILE;
     font_size = 16;
+    filename_font_size = 14;
+    dirname_font_size = 14;
+    user_font_size = 14;
     dir_colour       = vec3(1.0f);
     font_colour      = vec3(1.0f);
     highlight_colour = vec3(1.0f);
     selection_colour = vec3(1.0, 1.0, 0.3f);
 
     dir_name_depth = 0;
+    dir_name_position = 0.5f;
 
     elasticity = 0.0f;
 
@@ -445,12 +485,19 @@ void GourceSettings::setGourceDefaults() {
     file_show_filters.clear();
 
     file_extensions = false;
+    file_extension_fallback = false;
 
     //delete user filters
     for(std::vector<Regex*>::iterator it = user_filters.begin(); it != user_filters.end(); it++) {
         delete (*it);
     }
     user_filters.clear();
+
+    //delete user whitelist
+    for(std::vector<Regex*>::iterator it = user_show_filters.begin(); it != user_show_filters.end(); it++) {
+        delete (*it);
+    }
+    user_show_filters.clear();
 }
 
 void GourceSettings::commandLineOption(const std::string& name, const std::string& value) {
@@ -644,10 +691,25 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         auto_skip_seconds = -1.0;
     }
 
+    if(gource_settings->getBool("disable-input")) {
+        disable_input=true;
+    }
+
     if(gource_settings->getBool("loop")) {
         loop = true;
     }
 
+    if((entry = gource_settings->getEntry("loop-delay-seconds")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify loop-delay-seconds (float)");
+
+        loop_delay_seconds = entry->getFloat();
+
+        if(loop_delay_seconds<=0.0f) {
+            conffile.invalidValueException(entry);
+        }
+    }
+
     if((entry = gource_settings->getEntry("git-branch")) != 0) {
 
         if(!entry->hasValue()) conffile.missingValueException(entry);
@@ -894,6 +956,17 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         }
     }
 
+    if((entry = gource_settings->getEntry("font-file")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify font file");
+
+        font_file = entry->getString();
+
+        if(font_file.empty()) {
+           conffile.invalidValueException(entry);
+        }
+    }
+
     if((entry = gource_settings->getEntry("font-size")) != 0) {
 
         if(!entry->hasValue()) conffile.entryException(entry, "specify font size");
@@ -905,6 +978,54 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         }
     }
 
+    if((entry = gource_settings->getEntry("file-font-size")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify font size");
+
+        filename_font_size = entry->getInt();
+
+        if(filename_font_size<1 || filename_font_size>100) {
+            conffile.invalidValueException(entry);
+        }
+    }
+
+    if((entry = gource_settings->getEntry("dir-font-size")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify font size");
+
+        dirname_font_size = entry->getInt();
+
+        if(dirname_font_size<1 || dirname_font_size>100) {
+            conffile.invalidValueException(entry);
+        }
+    }
+
+    if((entry = gource_settings->getEntry("user-font-size")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify font size");
+
+        user_font_size = entry->getInt();
+
+        if(user_font_size<1 || user_font_size>100) {
+            conffile.invalidValueException(entry);
+        }
+    }
+
+    if((entry = gource_settings->getEntry("font-scale")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify font scale");
+
+        float font_scale = entry->getFloat();
+
+        if(font_scale<0.0f || font_scale>10.0f) {
+            conffile.invalidValueException(entry);
+        }
+
+        font_size         = glm::clamp((int)(font_size * font_scale), 1, 100);
+        user_font_size    = glm::clamp((int)(user_font_size * font_scale), 1, 100);
+        dirname_font_size = glm::clamp((int)(dirname_font_size * font_scale), 1, 100);
+    }    
+
     if((entry = gource_settings->getEntry("hash-seed")) != 0) {
 
         if(!entry->hasValue()) conffile.entryException(entry, "specify hash seed (integer)");
@@ -1194,6 +1315,10 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         days_per_second = 1.0 / 86400.0;
     }
 
+    if(gource_settings->getBool("no-time-travel")) {
+        no_time_travel = true;
+    }
+
     if(gource_settings->getBool("dont-stop")) {
         dont_stop = true;
     }
@@ -1329,6 +1454,10 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         file_extensions=true;
     }
 
+    if(gource_settings->getBool("file-extension-fallback")) {
+        file_extension_fallback=true;
+    }
+
     if((entry = gource_settings->getEntry("file-filter")) != 0) {
 
         ConfEntryList* filters = gource_settings->getEntries("file-filter");
@@ -1398,6 +1527,29 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         }
     }
 
+    if((entry = gource_settings->getEntry("user-show-filter")) != 0) {
+
+        ConfEntryList* filters = gource_settings->getEntries("user-show-filter");
+
+        for(ConfEntryList::iterator it = filters->begin(); it != filters->end(); it++) {
+
+            entry = *it;
+
+            if(!entry->hasValue()) conffile.entryException(entry, "specify user-show-filter (regex)");
+
+            std::string filter_string = entry->getString();
+
+            Regex* r = new Regex(filter_string, 1);
+
+            if(!r->isValid()) {
+                delete r;
+                conffile.entryException(entry, "invalid user-show-filter regular expression");
+            }
+
+            user_show_filters.push_back(r);
+        }
+    }
+
     if((entry = gource_settings->getEntry("dir-name-depth")) != 0) {
 
         if(!entry->hasValue()) conffile.entryException(entry, "specify dir-name-depth (depth)");
@@ -1409,6 +1561,17 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         }
     }
 
+    if((entry = gource_settings->getEntry("dir-name-position")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify dir-name-position (float)");
+
+        dir_name_position = entry->getFloat();
+
+        if(dir_name_position < 0.1f || dir_name_position > 1.0f) {
+            conffile.entryException(entry, "dir-name-position outside of range 0.1 - 1.0 (inclusive)");
+        }
+    }
+
     //validate path
     if(gource_settings->hasValue("path")) {
         path = gource_settings->getString("path");
