@@ -18,7 +18,7 @@
 #ifndef GOURCE_SETTINGS_H
 #define GOURCE_SETTINGS_H
 
-#define GOURCE_VERSION "0.49"
+#define GOURCE_VERSION "0.50"
 
 #include "core/texture.h"
 #include "core/settings.h"
@@ -44,6 +44,8 @@ public:
 
     bool disable_auto_rotate;
 
+    bool disable_input;
+
     bool show_key;
 
     std::string load_config;
@@ -67,10 +69,12 @@ public:
     bool stop_on_idle;
     bool stop_at_end;
     bool dont_stop;
+    bool no_time_travel;
 
     float auto_skip_seconds;
     float days_per_second;
     float file_idle_time;
+    float loop_delay_seconds;
 
     bool loop;
 
@@ -99,7 +103,11 @@ public:
 
     std::string title;
 
+    std::string font_file;
     int font_size;
+    int filename_font_size;
+    int dirname_font_size;
+    int user_font_size;
     vec3 font_colour;
 
     float elasticity;
@@ -126,13 +134,16 @@ public:
     vec3 selection_colour;
 
     int dir_name_depth;
+    float dir_name_position;
 
     std::vector<std::string> highlight_users;
     std::vector<std::string> follow_users;
     std::vector<Regex*> file_filters;
     std::vector<Regex*> file_show_filters;
     std::vector<Regex*> user_filters;
+    std::vector<Regex*> user_show_filters;
     bool file_extensions;
+    bool file_extension_fallback;
 
     std::string caption_file;
     vec3 caption_colour;
